/*
** ATOP - System & Process Monitor 
**
** The program 'atop' offers the possibility to view the activity of
** the system on system-level as well as process-level.
**
** This source-file contains the Linux-specific functions to calculate
** figures to be visualized.
** ================================================================
** Author:      Gerlof Langeveld - AT Computing, Nijmegen, Holland
** E-mail:      gerlof@ATComputing.nl
** Date:        July 2002
** 
** This program is free software; you can redistribute it and/or modify it
** under the terms of the GNU General Public License as published by the
** Free Software Foundation; either version 2, or (at your option) any
** later version.
** 
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
** See the GNU General Public License for more details.
**
** $Log: showlinux.c,v $
** Revision 1.9  2003/01/24 14:20:57  gerlof
** If possible, also show commandline when process has exited.
**
** Revision 1.8  2003/01/17 07:32:49  gerlof
** Show the full command-line per process (option 'c').
**
** Revision 1.7  2002/10/04 10:05:54  gerlof
** Bug-solution: New process indicator in static output set when needed.
**
** Revision 1.6  2002/10/03 11:14:42  gerlof
** Modify (effective) uid/gid to real uid/gid.
**
** Revision 1.5  2002/09/26 13:52:51  gerlof
** Limit header lines by not showing disks.
** Limit header lines by not showing disks.
**
** Revision 1.4  2002/09/16 08:59:13  gerlof
** Change field EXCODE to STATUS for support of indicator of newly created
** processes.
**
** Revision 1.3  2002/09/02 08:42:44  gerlof
** Bug-solution: blank line after header when more than 999 screens of
** process-list information.
**
** Revision 1.2  2002/08/30 07:11:20  gerlof
** Minor changes in the header-line of the process list.
**
** Revision 1.1  2002/07/24 11:14:16  gerlof
** Initial revision
**
**
** Initial
**
*/

static char rcsid[] = "$Id: showlinux.c,v 1.9 2003/01/24 14:20:57 gerlof Exp $";

#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <signal.h>
#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <fcntl.h>
#include <termio.h>
#include <unistd.h>
#include <stdarg.h>
#include <curses.h>
#include <pwd.h>
#include <grp.h>

#include "atop.h"
#include "photoproc.h"
#include "photosyst.h"
#include "showgeneric.h"

extern int      acctfd;
extern char     kernpatch;
extern time_t   pretime;        /* previous sample time          */


/*
** format-lines for printing
*/
char 	*prcline   	= "PRC | sys %s | user %s | #procs %s | #exits %s |"
	       	     	  " #zombie %s |\n";


char	*genprochdr1	= "\n  PID  SYSCPU  USRCPU  VGROW  RGROW  USERNAME  PRI"
		     	  " STATUS S  %s CMD  %4d/%-4d\n";
char	*genprochdr2	= "\n  PID  SYSCPU  USRCPU  VGROW  RGROW  READ WRITE "
		     	  "RNET SNET S  %s CMD  %4d/%-4d\n";

char 	*genproclina1	= "%5d %s %s %s %s  %-9.9s %3d %-6s %c %3.0f%% "
	                  "%.14s\n";
char 	*genproclina2	= "%5d %s %s %s %s %s %s %s %s %c %3.0f%% %.14s\n";

char 	*genprocline1	= "%s %s                %-9.9s     %s"
			  " E %3.0f%% <%.12s>\n";
char 	*genprocline2	= "%s %s               (r&w %s)        "
			  "  E %3.0f%% <%.12s>\n";


char	*memprochdr	= "\n  PID MINFLT MAJFLT #SWP VSTEXT "
		     	  " VSIZE  RSIZE  VGROW  RGROW  %s CMD  %4d/%-4d\n";
char 	*memproclina	= "%5d %s %s %s %s %s %s %s %s %3.0f%% "
	                  "%.14s\n";
char 	*memprocline	= "%s %s %s                                  "
			  "  %3.0f%% <%.12s>\n";


char	*netprochdr	= "\n  PID   TCPRCV  TCPSND    UDPRCV  UDPSND"
		     	  "                     %s CMD  %4d/%-4d\n";
char 	*netproclina	= "%5d  %s %s   %s %s                    %3.0f%% "
	                  "%.14s\n";
char 	*netprocline	= "                                        "
			  "              %3.0f%% <%.12s>\n";


char	*staprochdr	= "\n  PID  USERNAME GROUP       STDATE    STTIME      "
	                  " STATUS S  %s CMD  %4d/%-4d\n";
char 	*staproclina	= "%5d  %-8.8s %-8.8s  %s %s      %s    %c %3.0f%% "
	                  "%.14s\n";
char 	*staprocline	= " %-8.8s %-8.8s  %s %s      %s E %3.0f%% <%.12s>\n";


char	*cmdprochdr	= "\n  PID  %s COMMAND-LINE                           "
			  "                    %4d/%-4d\n";
char 	*cmdproclina	= "%5d %3.0f%% %s\n";
char 	*cmdprocline	= "%3.0f%% <%.66s>\n";

/*
** calculate the total consumption on system-level for the 
** four main resources
*/
void
totalcap(struct syscap *psc, struct sstat *sstat)
{
	unsigned int    pgsz = getpagesz(), i;

	psc->availcpu	= sstat->cpu.stime + sstat->cpu.utime +
			  sstat->cpu.ntime + sstat->cpu.itime;
	psc->availmem	= sstat->mem.physmem * pgsz/1024;
	psc->availnet	= sstat->net.tcpin + sstat->net.tcpout +
	                  sstat->net.udpin + sstat->net.udpout;

	/*
	** calculate total number of accesses which have been
	** issued by the active disks
	*/
	psc->availdsk = 0;

	for (i=0; sstat->xdsk[i].name[0]; i++)
		psc->availdsk += sstat->xdsk[i].nread+sstat->xdsk[i].nwrite;
}

/*
** calculate cumulative system- and user-time for all active processes
*/
void
pricumproc(struct pstat *pstat, int nact, int nproc, int nzomb, int nexit)
{
	unsigned long	i, totut, totst;
        char            format1[16], format2[16], format3[16];
	char            format4[16], format5[16];

	for (i=totut=totst=0; i < nact; i++)
	{
		totut += (pstat+i)->cpu.utime;
		totst += (pstat+i)->cpu.stime;
	}

	printg(prcline, val2cpustr(totst * 1000/HZ, format1),
			val2cpustr(totut * 1000/HZ, format2),
			val2valstr(nproc,           format3, 5),
			acctfd >= 0 ?
			          val2valstr(nexit, format4, 5) : "    ?",
			val2valstr(nzomb,           format5, 4));
}

/*
** print a line of general figures about one process
*/
static char	*sigs[] = { "S_SIG0", "S_HUP",  "S_INT",  "S_QUIT", "S_ILL",
		            "S_TRAP", "S_IOT",  "S_BUS",  "S_FPE",  "S_KILL",
		            "S_USR1", "S_SEGV", "S_USR2", "S_PIPE", "S_ALRM",
		            "S_TERM", "S_STKF", "S_CHLD", "S_CONT", "S_STOP",
		            "S_TSTP", "S_TTIN", "S_TTOU", "S_URG",  "S_XCPU",
		            "S_XFSZ", "S_VALM", "S_PROF", "S_WINC", "S_POLL",
			    "S_PWR",  "S_SYS", };

void
showgenproc(struct pstat *curstat, double perc)
{
	struct passwd 	*pwd;
	char		*username, curstate;

	char		format1[16], format2[16];
	char		format3[16], format4[16];
	char		format5[16], format6[16];
	char		format7[16], format8[16];

	if (curstat->gen.state != 'E')
	{
		/*
		** show process-info of active process
		*/
		if (!kernpatch)
		{
			/*
			** determine the user-name of process
			*/
			if ( (pwd = getpwuid(curstat->gen.ruid)) == NULL)
				username = "unknown";
			else
				username = pwd->pw_name;

			printg(genproclina1,
			   curstat->gen.pid,
			   val2cpustr(curstat->cpu.stime*1000/HZ, format1),
			   val2cpustr(curstat->cpu.utime*1000/HZ, format2),
			   val2memstr(curstat->mem.vgrow*1024,    format3,
								  KBFORMAT),
			   val2memstr(curstat->mem.rgrow*1024,    format4,
								  KBFORMAT),
			   username,
			   curstat->cpu.pri,
			   curstat->gen.excode & ~(LONG_MAX) ? "N--" : "---",
			   curstat->gen.state, perc, curstat->gen.name);
		}
		else
		{
			printg(genproclina2,
			   curstat->gen.pid,
			   val2cpustr(curstat->cpu.stime*1000/HZ, format1),
			   val2cpustr(curstat->cpu.utime*1000/HZ, format2),
			   val2memstr(curstat->mem.vgrow*1024,    format3,
								  KBFORMAT),
			   val2memstr(curstat->mem.rgrow*1024,    format4,
								  KBFORMAT),
			   val2valstr(curstat->dsk.ior,           format5, 5),
			   val2valstr(curstat->dsk.iow,           format6, 5),
			   val2valstr(curstat->net.tcprcv+curstat->net.udprcv,
			                                          format7, 4),
			   val2valstr(curstat->net.tcpsnd+curstat->net.udpsnd,
			                                          format8, 4),
			   curstat->gen.state, perc, curstat->gen.name);
		}
	}
	else
	{
		/*
		** show process-info of exited process
		*/
		if (curstat->gen.pid == 0)
			printg("    ? ");
		else
			printg("%5d ", curstat->gen.pid);

		if (!kernpatch)
		{
			if ( (pwd = getpwuid(curstat->gen.ruid)) == NULL)
				username = "unknown";
			else
				username = pwd->pw_name;

			sprintf(format3, "%c%c%c%3d",
				curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
		  		curstat->gen.excode & 0xff        ? 'S' : 'E',
		  		curstat->gen.excode & 0x80        ? 'C' : '-',
		  		curstat->gen.excode & 0xff        ? 
					 curstat->gen.excode     & 0x7f :
			  		(curstat->gen.excode>>8) & 0xff );

			printg(genprocline1,
			  val2cpustr(curstat->cpu.stime*1000/HZ, format1),
			  val2cpustr(curstat->cpu.utime*1000/HZ, format2),
			  username,
			  format3,
			  perc, curstat->gen.name);
		}
		else
		{
			printg(genprocline2,
			  val2cpustr(curstat->cpu.stime*1000/HZ, format1),
			  val2cpustr(curstat->cpu.utime*1000/HZ, format2),
			  val2valstr(curstat->dsk.ior, format3, 6),
			  perc, curstat->gen.name);
		}
	}
}

/*
** print a line of memory-figures about one process
*/
void
showmemproc(struct pstat *curstat, double perc)
{
	char	format1[16], format2[16];
	char	format3[16], format4[16];
	char	format5[16], format6[16];
	char	format7[16], format8[16];

	if (curstat->gen.state != 'E')
	{
		/*
		** show process-info of active process
		*/
		printg(memproclina,
		  curstat->gen.pid,
		  val2valstr(curstat->mem.minflt,      format1, 6),
		  val2valstr(curstat->mem.majflt,      format2, 6),
		  val2valstr(curstat->mem.iosw,        format3, 4),
		  val2memstr(curstat->mem.shtext*1024, format4, KBFORMAT),
		  val2memstr(curstat->mem.vmem*1024,   format5, KBFORMAT),
		  val2memstr(curstat->mem.rmem*1024,   format6, KBFORMAT),
		  val2memstr(curstat->mem.vgrow*1024,  format7, KBFORMAT),
		  val2memstr(curstat->mem.rgrow*1024,  format8, KBFORMAT),
		  perc, curstat->gen.name);
	}
	else
	{
		/*
		** show process-info of exited process
		*/
		if (curstat->gen.pid == 0)
			printg("    ? ");
		else
			printg("%5d ", curstat->gen.pid);

		printg(memprocline,
		  val2valstr(curstat->mem.minflt,     format1, 6),
		  val2valstr(curstat->mem.majflt,     format2, 6),
		  val2valstr(curstat->mem.iosw,       format3, 4),
		  perc,
		  curstat->gen.name);
	}
}

/*
** print a line of network-figures about one process
*/
void
shownetproc(struct pstat *curstat, double perc)
{
	char	format1[16], format2[16];
	char	format3[16], format4[16];

	if (curstat->gen.state != 'E')
	{
		/*
		** show process-info of active process
		*/
		printg(netproclina,
		  curstat->gen.pid,
		  val2valstr(curstat->net.tcprcv,      format1, 7),
		  val2valstr(curstat->net.tcpsnd,      format2, 7),
		  val2valstr(curstat->net.udprcv,      format3, 7),
		  val2valstr(curstat->net.udpsnd,      format4, 7),
		  perc, curstat->gen.name);
	}
	else
	{
		/*
		** show process-info of exited process
		*/
		if (curstat->gen.pid == 0)
			printg("    ? ");
		else
			printg("%5d ", curstat->gen.pid);

		printg(netprocline,
		  perc,
		  curstat->gen.name);
	}
}

/*
** print a line of (more or less) static info about one process
*/
void
showstaproc(struct pstat *curstat, double perc)
{
	char		format1[16], format2[16], format3[16];

	struct passwd 	*pwd;
	struct group 	*grp;
	char		*username, *groupname;

	/*
	** determine the user- and group-name of the process
	*/
	if ( (pwd = getpwuid(curstat->gen.ruid)) == NULL)
		username = "unknown";
	else
		username = pwd->pw_name;

	if ( (grp = getgrgid(curstat->gen.rgid)) == NULL)
		groupname = "unknown";
	else
		groupname = grp->gr_name;

	/*
	** show process-info
	*/
	if (curstat->gen.state != 'E')
	{
		/*
		** show process-info of active process
		*/
		printg(staproclina,
		  	curstat->gen.pid,
		  	username,
		  	groupname,
			convdate(curstat->gen.btime, format1),
			convtime(curstat->gen.btime, format2),
			curstat->gen.excode & ~(LONG_MAX) ? "N--" : "---",
		  	curstat->gen.state,
		  	perc, curstat->gen.name);
	}
	else
	{
		/*
		** show process-info of exited process
		*/
		if (curstat->gen.pid == 0)
			printg("    ? ");
		else
			printg("%5d ", curstat->gen.pid);

		sprintf(format3, "%c%c%c%3d",
			curstat->gen.excode & ~(LONG_MAX) ? 'N' : '-',
		  	curstat->gen.excode & 0xff        ? 'S' : 'E',
		  	curstat->gen.excode & 0x80        ? 'C' : '-',
		  	curstat->gen.excode & 0xff        ? 
				 curstat->gen.excode     & 0x7f :
		  		(curstat->gen.excode>>8) & 0xff );

		printg(staprocline,
		  	username,
		  	groupname,
			convdate(curstat->gen.btime, format1),
			convtime(curstat->gen.btime, format2),
		  	format3,
		  	perc,
		  	curstat->gen.name);
	}
}

/*
** print a line containing the (almost) full command line
*/
void
showcmdproc(struct pstat *curstat, double perc)
{
	char		format1[16], format2[16], format3[16];

	/*
	** show process-info
	*/
	if (curstat->gen.state != 'E')
	{
		/*
		** show process-info of active process
		*/
		printg(cmdproclina,
		  	curstat->gen.pid,
			perc,
			curstat->gen.cmdline[0] ? 
			         curstat->gen.cmdline : curstat->gen.name);
	}
	else
	{
		/*
		** show process-info of exited process
		*/
		if (curstat->gen.pid == 0)
			printg("    ? ");
		else
			printg("%5d ", curstat->gen.pid);

		printg(cmdprocline,
			perc, 
			curstat->gen.cmdline[0] ? 
			         curstat->gen.cmdline : curstat->gen.name);
	}
}


/*
** print the header for the process-list
*/
void
priphead(int curlist, int totlist, char *sortorder, char showtype)
{
	switch (showtype)
	{
	   case 'g':
		if (!kernpatch)
			printg(genprochdr1, sortorder, curlist, totlist);
		else
			printg(genprochdr2, sortorder, curlist, totlist);
		break;

	   case 'm':
		printg(memprochdr, sortorder, curlist, totlist);
		break;

	   case 'n':
		printg(netprochdr, sortorder, curlist, totlist);
		break;

	   case 's':
		printg(staprochdr, sortorder, curlist, totlist);
		break;

	   case 'c':
		printg(cmdprochdr, sortorder, curlist, totlist);
		break;
	}
}


/*
** print the list of processes from the deviation-list
*/
int
priproc(struct pstat  *pstat, int firstproc, int lastproc, int curline,
	int curlist, int totlist, char *sortorder,
	struct syscap *sb, char showtype,
	struct selection *sel, int screen)
{
	register int		i, j;
	register struct pstat	*curstat;
	double			perc;

	/*
	** print info per actual process and maintain totals
	*/
	for (i=firstproc; i < lastproc; i++)
	{
		if (screen && curline >= LINES)	/* screen filled entirely ? */
			break;

		curstat = pstat+i;

		/*
		** calculate occupation-percentage of this process
		** depending on selected resource
		*/
		switch (*sortorder) 
		{
		   case 'C':
			perc = (double)(curstat->cpu.stime+curstat->cpu.utime)
			        			 * 100.0 / sb->availcpu;
			break;

		   case 'M':
			perc = (double)(curstat->mem.rmem)
                                                         * 100.0 / sb->availmem;
			break;

		   case 'D':
			perc = (double)(curstat->dsk.ior + curstat->dsk.iow)
			        			 * 100.0 / sb->availdsk;
			break;

		   case 'N':
			perc = (double)(curstat->net.tcpsnd +
			                curstat->net.tcprcv +
			                curstat->net.udpsnd +
			                curstat->net.udprcv  )
			        		         * 100.0 / sb->availnet;
			break;

		   default:
			perc = 0.0;
		}

		if (perc > 100.0)
			perc = 100.0;

		/*
		** check if only the processes of a particular user
		** should be shown
		*/
		if (sel->userid != ALLUSERS && sel->userid != curstat->gen.ruid)
			continue;

		/*
		** check if only the processes with a particular name
		** should be shown
		*/
		if (sel->procnamesz &&
		    memcmp(sel->procname, curstat->gen.name, sel->procnamesz) )
			continue;

		/*
		** now do the formatting of output
		*/
		switch (showtype)
		{
		   case 'g':
			showgenproc(curstat, perc);
			break;

		   case 'm':
			showmemproc(curstat, perc);
			break;

		   case 'n':
			shownetproc(curstat, perc);
			break;

		   case 's':
			showstaproc(curstat, perc);
			break;

		   case 'c':
			showcmdproc(curstat, perc);
			break;
		}

		curline++;
	}

	return curline;
}


/*
** print the system-wide statistics
*/
int
prisyst(struct sstat *sstat, int curline, int nsecs,
	 			int fixedhead, int limithead)
{
	register int	i;
	double		cputot, mstot;
	char		format1[16], format2[16], format3[16];
	char		format4[16], format5[16], format6[16];
	unsigned int	pgsz = getpagesz();

	/*
	** CPU statistics
	*/
	cputot = (double) sstat->cpu.stime + sstat->cpu.utime +
	                  sstat->cpu.ntime + sstat->cpu.itime;

	if (cputot == 0.0)
		cputot = 1.0;		/* avoid divide-by-zero */

	printg("CPU | system %3.0f%% | user    %3.0f%% | nice %6.0f%% | "
	       "idle %6.0f%% | cs %s |\n",
			(double) (sstat->cpu.stime * 100.0) / cputot,
			(double) (sstat->cpu.utime * 100.0) / cputot,
			(double) (sstat->cpu.ntime * 100.0) / cputot,
			(double) (sstat->cpu.itime * 100.0) / cputot,
			val2valstr(sstat->cpu.csw, format1, 9));

	curline++;

	/*
	** MEMORY statistics
	*/
	printg("MEM | tot  %s | free  %s | cache %s | buff  %s | share %s |\n",
		val2memstr(sstat->mem.physmem  * pgsz, format1, MBFORMAT),
		val2memstr(sstat->mem.freemem  * pgsz, format2, MBFORMAT),
		val2memstr(sstat->mem.cachemem * pgsz, format3, MBFORMAT),
		val2memstr(sstat->mem.buffermem* pgsz, format4, MBFORMAT),
		val2memstr(sstat->mem.sharedmem* pgsz, format5, MBFORMAT));

	curline++;

	/*
	** SWAP statistics
	*/
	printg("SWP | tot  %s | free  %s |              | "
	       "swin %s | swout %s |\n",
		val2memstr(sstat->mem.totswap  * pgsz, format1, MBFORMAT),
		val2memstr(sstat->mem.freeswap * pgsz, format2, MBFORMAT),
		val2valstr(sstat->mem.swins,           format3, 7),
		val2valstr(sstat->mem.swouts,          format4, 6));

	curline++;

	/*
	** DISK statistics
	**
	** If extended disk-stats's are supported, take them
	*/
	mstot = cputot * 1000.0 / HZ / sstat->cpu.numcpu; 

	if (!limithead)
	{
		for (i=0; sstat->xdsk[i].name[0]; i++)
		{
			unsigned long 	iotot;

			iotot =  sstat->xdsk[i].nread + sstat->xdsk[i].nwrite;

			if (iotot || fixedhead)
			{
			    printg("DSK | %11.11s | busy %6.0lf%% | read %s |"
			           " write %s | avio %4.0lf ms |\n",
			    	sstat->xdsk[i].name,
				sstat->xdsk[i].io_ms * 100.0 / mstot,
			    	val2valstr(sstat->xdsk[i].nread,  format1, 7),
			    	val2valstr(sstat->xdsk[i].nwrite, format2, 6),
			    	iotot ?
				   (double)sstat->xdsk[i].io_ms/iotot : 0.0);

			    curline++;
			}
		}
	}

	/*
	** NET statistics
	*/
	if (sstat->net.tcpin || sstat->net.tcpout ||
	    sstat->net.udpin || sstat->net.udpout || fixedhead )
	{
		printg("NET |   transport | tcpi %s | tcpo %s |"
		       " udpi %s | udpo %s |\n",
			val2valstr(sstat->net.tcpin,   format1, 7),
			val2valstr(sstat->net.tcpout,  format2, 7),
			val2valstr(sstat->net.udpin,   format3, 7),
			val2valstr(sstat->net.udpout,  format4, 7));

		curline++;
	}

	if (sstat->net.ipin || sstat->net.ipout || fixedhead )
	{
		printg("NET |     network | ipi %s | ipo %s | ipfrw %s |"
		       " deliv %s |\n",
			val2valstr(sstat->net.ipin,    format1, 8),
			val2valstr(sstat->net.ipout,   format2, 8),
			val2valstr(sstat->net.ipfrw,   format3, 6),
			val2valstr(sstat->net.ipindel, format4, 6));

		curline++;
	}

	for (i=0; sstat->intf[i].name[0]; i++)
	{
		if (sstat->intf[i].rpack || sstat->intf[i].spack || fixedhead)
		{
			printg("NET | dev %7.7s | pcki %s | pcko %s |"
			       " in %sK | out %sK |\n",
			    	sstat->intf[i].name,
				val2valstr(sstat->intf[i].rpack, format1, 7),
				val2valstr(sstat->intf[i].spack, format2, 7),
				val2valstr(sstat->intf[i].rbyte/1024,
				                                 format3, 8),
				val2valstr(sstat->intf[i].sbyte/1024,
				                                 format4, 7));

			curline++;
		}
	}

	return curline;
}

/*
** sort-functions
*/
int
compcpu(const void *a, const void *b)
{
	register ulong	acpu = ((struct pstat *)a)->cpu.stime +
			       ((struct pstat *)a)->cpu.utime;
	register ulong	bcpu = ((struct pstat *)b)->cpu.stime +
			       ((struct pstat *)b)->cpu.utime;

	if (acpu < bcpu)
		return(1);
	if (acpu > bcpu)
		return(-1);

	return compmem(a, b);
}

int
compdsk(const void *a, const void *b)
{
	int		i;
	register ulong	adsk = ((struct pstat *)a)->dsk.ior +
			       ((struct pstat *)a)->dsk.iow;
	register ulong	bdsk = ((struct pstat *)b)->dsk.ior +
			       ((struct pstat *)b)->dsk.iow;

	if (adsk < bdsk)
		return(1);
	if (adsk > bdsk)
		return(-1);

	return compcpu(a, b);
}

int
compmem(const void *a, const void *b)
{
	register ulong	amem = ((struct pstat *)a)->mem.rmem;
	register ulong	bmem = ((struct pstat *)b)->mem.rmem;

	if (amem < bmem)
		return(1);
	if (amem > bmem)
		return(-1);
	return 0;
}

int
compnet(const void *a, const void *b)
{
	int		i;
	register ulong	anet = ((struct pstat *)a)->net.tcpsnd +
			       ((struct pstat *)a)->net.tcprcv +
			       ((struct pstat *)a)->net.udpsnd +
			       ((struct pstat *)a)->net.udprcv  ;
	register ulong	bnet = ((struct pstat *)b)->net.tcpsnd +
			       ((struct pstat *)b)->net.tcprcv +
			       ((struct pstat *)b)->net.udpsnd +
			       ((struct pstat *)b)->net.udprcv  ;

	if (anet < bnet)
		return(1);
	if (anet > bnet)
		return(-1);

	return compcpu(a, b);
}
